/*
 * AuxProcess.java
 *
 * Utility class for 
 * 1. running background processes. 
 * 2. maintaining a name for the current JVM process
 */

package util;

import java.io.*;

public class AuxProcess {

    // Run command, echoing its stdout and stderr output streams to the
    // main process's stdout and stderr streams.

    // Return process object.  Useful for later killing process, if process
    // isn't expected to terminate by itself.

    public static Process exec(String prefix, String command)
            throws IOException {

        Process proc = Runtime.getRuntime().exec(command);
        echo(proc.getInputStream(), "[" + prefix + " stdout]: ", System.out);
        echo(proc.getErrorStream(), "[" + prefix + " stderr]: ", System.err);

        return proc;
    }
    // Kill process if exists

    public static void shutDown(Process p) {
        if (p != null) p.destroy();
    }
    
    // Start background thread to read lines from InputStream
    // and echo to a PrintStream with a provided prefix.

    private static void echo(final InputStream is,
                             final String prefix,
                             final PrintStream ps) {
        Thread t = new Thread(new Runnable() 
        {
            public void run() {
                InputStreamReader isr = new InputStreamReader(is);
                BufferedReader in = new BufferedReader(isr);
                try {
                    String s = in.readLine();
                    while (s != null) {
                        ps.println(prefix + s);
                        s = in.readLine();
                    }
                }
                catch (IOException e) {
                    ps.println(prefix +
                               "Auxprocess.echo() caught IOException");
                    e.printStackTrace(ps);
                    return;
                }
            }
            });
        t.start();
    }

    // Maintain a name for the current process.
    
    private static String name = "";
    
    public static void setJVMName(String s) { name = s;}
    public static String getJVMName() {return name;}

}
