/**
 * CNFactoryImp.java
 * 
 * @author Paul Jackson
 * 
 */
package dhash;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import java.util.logging.*;

import util.AuxProcess;

public class CNFactoryImp 
             implements CNFactory {
        
    //  See Serializable API docs. UnicastRemoteObject implements it.
    private static final long serialVersionUID = 42L;

    // Ensure this is a singleton class.
        
    private CNFactoryImp() throws RemoteException {};
        
    private static CNFactoryImp factoryObj;
    static {
        try {
            factoryObj = new CNFactoryImp();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    // Get factory object.  Useful when want to make local CNSlave objects.

    public static CNFactory getFactory() {
        return factoryObj;
    }

    // Set ID size.  

    public void setIDSize(int i) {
        ID.setSize(i);
    }

    /* 
     * 
     */
    public CNSlave makeCNSlave(String name, ID id) 
        throws RemoteException
    {
        return new CNSlaveImp(name, AuxProcess.getJVMName(), id);
    }

    /**
     * Create and export a ChordNode factory for remote access
     */
    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Domain name must be supplied as argument");
            System.exit(0);
        }
        String domName = args[0];
        AuxProcess.setJVMName(domName);

        try {
            // The 0 as 2nd arg of exportObject specifies an anonymous port:
            // Port number will be chosen at runtime.
	    CNFactory factoryObjStub = (CNFactory)
                UnicastRemoteObject.exportObject(factoryObj, 0);
            
            Registry registry = LocateRegistry.getRegistry();  
            registry.bind("cn-factory/dom" + domName, factoryObjStub);
            
        } catch (Exception e) {
            System.out.println("CNFactoryImp dom" + domName
                               + " exception: " + e.getMessage());
            e.printStackTrace();
        }

        // Enable handling of all log records by root handler.
        Logger.getLogger("").getHandlers()[0].setLevel(Level.ALL);        

        // Enable logging
        // Logger.getLogger("dhash.ChordNodeImp").setLevel(Level.FINE);
        // Logger.getLogger("dhash.ChordNodeImp").setLevel(Level.FINER);

        
    }
}
