/**
 * CNSlaveImp.java
 */

package dhash;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * @author Paul Jackson
 * 
 */
public class CNSlaveImp 
    extends UnicastRemoteObject 
    implements CNSlave, AppNode {

    //== Static fields ======================================================

    private static final long serialVersionUID = 42L;

    //== Instance fields ====================================================

    private ChordNodeImp cNode;

    //== Constructors =======================================================

    CNSlaveImp(String name, String domain, ID id) throws RemoteException {
        super(); // Export object so available to receive RMI calls.
                 // javac supplies call if not explicit.  Made explicit
                 // here for clarity.
        cNode = new ChordNodeImp(name, domain, id);
        cNode.setAppNode(this);
    }

    //== Methods for controlling ring structure =============================

    /*
     */
    public void initForNewRing() {
        cNode.initForNewRing();
    }

    /*
     */
    public void join(CNSlave nprime) throws RemoteException {
        cNode.join(nprime.getChordNode());
    }

    //== Main Chord protocol lookup method ==================================

    /*
     */
    public CNSlave findSuccessor(ID id) throws RemoteException {
        ChordNodeRef succNode = cNode.findSuccessor(id);
        return (CNSlave) succNode.getAppNode();

    }
    

    //== CNSlave utility methods ============================================

    public ChordNodeRef getChordNode() {
        return new ChordNodeRef(cNode);
    }

    public ID getID() throws RemoteException {
        return cNode.getID();
    }

    public String toString() { 
        return "CNSlave [" + cNode.toString() + "]";
    }
    
    public String toLongString() throws RemoteException {
        return cNode.toLongString();
    }

    //== AppNode interface methods ===========================================
    /*
     * Call back method from ChordNode.  Does nothing for now. 
     * Should engineer eventually so that testbench checks
     * proper calling of this method.
     */
    public void processNewPredecessor(){
        return;
    }

}
