/**
 * DHNFactoryImp.java
 * 
 * 
 * @author Paul Jackson
 * 
 */
package dhash;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import java.util.logging.*;

import util.AuxProcess;

public class DHNFactoryImp 
             implements DHNFactory {
        
    //  See Serializable API docs. UnicastRemoteObject implements it.
    private static final long serialVersionUID = 42L;

    // Ensure this is a singleton class.
        
    private DHNFactoryImp() throws RemoteException {};
        
    private static DHNFactoryImp factoryObj;
    static {
        try {
            factoryObj = new DHNFactoryImp();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
    
    // Get factory object.  Useful when want to make local DHNClient objects.

    public static DHNFactory getFactory() {
        return factoryObj;
    }

    // Set ID size.  

    public void setIDSize(int i) {
        ID.setSize(i);
    }

    public void setLoggingLevel(String loggerName, Level level) {
        Logger.getLogger(loggerName).setLevel(level);
    }

    /* 
     * 
     */
    public DHNSlave makeDHNSlave(String name, ID id) 
        throws RemoteException
    {
        return new DHNSlaveImp(name, AuxProcess.getJVMName(), id);
    }
    public DHNSlave makeDHNSlave(String name) throws RemoteException {

        return new DHNSlaveImp(name, AuxProcess.getJVMName());
    }

    /**
     * Create and export a DHNode factory for remote access.
     */

    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Domain name must be supplied as argument");
            System.exit(0);
        }
        String domName = args[0];
        AuxProcess.setJVMName(domName);

        Monitor monitor;

        // Initialise remote reference to monitor object in master test process
        // Then pass factory reference to monitor.

        try {
            // The 0 as 2nd arg of exportObject specifies an anonymous port:
            // Port number will be chosen at runtime.

            // DELETE following if not needed.
            //	    DHNFactory factoryObjStub = (DHNFactory)

            UnicastRemoteObject.exportObject(factoryObj, 0);

            Registry registry = LocateRegistry.getRegistry();
            monitor = (Monitor) registry.lookup("dhash/monitor");
            monitor.put(domName, factoryObj);
            
        } catch (Exception e) {
            System.out.println("DHNFactoryImp dom" + domName
                               + " exception: " + e.getMessage());
            e.printStackTrace();
            return;
        }

        // Create a new handler for custom tracing of ChordNode methods
        Handler cNIHandler = new CNHandler(monitor);
        cNIHandler.setLevel(Level.ALL);

        // Install new handler and enable
        Logger cNILogger = Logger.getLogger("dhash.ChordNodeImp");
        cNILogger.addHandler(cNIHandler);
        cNILogger.setUseParentHandlers(false);

        return;
    }
}
