/*
 * DHNSlaveImp.java
 *
 */

package dhash;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 *
 * @author  Paul Jackson
 */

public class DHNSlaveImp 
       extends UnicastRemoteObject
       implements DHNSlave {

    //== Static fields ======================================================

    private static final long serialVersionUID = 42L;

    //== Instance fields ====================================================

    //    private DHashNodeImp dHNode;
    private DHashNodeClient dHNode;

    //== Constructors =======================================================

    DHNSlaveImp(String name, String domain, ID id) throws RemoteException {
        super(); // Export object so available to receive RMI calls.
                 // javac supplies call if not explicit.  Made explicit
                 // here for clarity.
        dHNode = new DHashNodeImp(name, domain, id);
        ((DHashNodeImp) dHNode).setDHAppNode(new SimpleDHAppImp());
            //dHNode.setDHAppNode(new SimpleDHAppImp());
    }

    DHNSlaveImp(String name, String domain) throws RemoteException {
        super();
        dHNode = new DHashNodeImp(name, domain);
        ((DHashNodeImp) dHNode).setDHAppNode(new SimpleDHAppImp());
    }

    //== Methods for controlling ring structure =============================

    public void initForNewRing() throws RemoteException {
        dHNode.initForNewRing();
    }

    public void join(DHNSlave dHNS) throws RemoteException {
        dHNode.join(dHNS.getDHashNode());
    }
    
    //== HashMap methods ====================================================
    public void put(String key, Object value) throws RemoteException {
        dHNode.put(key, value);
    }

    public Object get(String key) throws RemoteException {
        return dHNode.get(key);
    }
    
    public boolean contains(String key) throws RemoteException {
        return dHNode.contains(key);
    }
    
    public void remove(String key) throws RemoteException {
        dHNode.remove(key);
    }

    public DHAppNode dHAppGet(String key) throws RemoteException {
        return dHNode.dHAppGet(key);
    }

    //== Debug methods ======================================================
    public ChordNodeRef getChordNode() throws RemoteException {
        return ((DHashNodeImp) dHNode).getChordNode();
    }

    //== Slave utility methods ==============================================
    public DHashNode getDHashNode() throws RemoteException {
        return (DHashNode) dHNode;
    }
}
