/*
 * IDTest.java
 *
 * @author  Paul Jackson
 *
 * Illustrates use of JUnit 4.
 * See http://junit.sourceforge.net/
 *
 */

package dhash;

//  Junit 4 
import org.junit.*;  
import static org.junit.Assert.*;
import org.junit.runner.JUnitCore;

import java.math.BigInteger;

public class IDTest {
    
    public IDTest() {};
    
    public static void main(String[] args) {

        JUnitCore.main("dhash.IDTest");
    }

    private ID i0;
    private ID i1;
    private ID i2;
    private ID i3;
    private ID i4;
    private ID i5;
    private ID i6;
    private ID i7;

    @Before
    public void setUp() {
        ID.setSize(3);
        i0 = new ID("0");
        i1 = new ID("1");
        i2 = new ID("2");
        i3 = new ID("3");
        i4 = new ID("4");
        i5 = new ID("5");
        i6 = new ID("6");
        i7 = new ID("7");

    }

    @Test
    public void testGetSize() {
        assertEquals(ID.getSize(), 3);
    }

    @Test
    public void testEquals() {
        assertTrue(i1.equals(i1));
        assertFalse(i2.equals(i3));
        assertFalse(i3.equals(i2));
    }

    @Test
    public void testDist() {
        assertEquals(i1.dist(i4), new BigInteger("3"));
        assertEquals(i5.dist(i2), new BigInteger("5"));
        assertEquals(i2.dist(i2), new BigInteger("0"));
    }
    @Test
    public void testAddPowerTwo() {
        assertEquals(i2, i1.addPowerTwo(0));
        assertEquals(i0, i7.addPowerTwo(0));
        assertEquals(i3, i1.addPowerTwo(1));
        assertEquals(i2, i0.addPowerTwo(1));
        assertEquals(i5, i1.addPowerTwo(2));
        assertEquals(i1, i5.addPowerTwo(2));
    }
    @Test
    public void testInOpenClosedStarRange() {
        assertFalse(i0.inOpenClosedStarRange(i1, i4)); 
        assertFalse(i1.inOpenClosedStarRange(i1, i4)); 
        assertTrue (i2.inOpenClosedStarRange(i1, i4)); 
        assertTrue (i3.inOpenClosedStarRange(i1, i4)); 
        assertTrue (i4.inOpenClosedStarRange(i1, i4)); 
        assertFalse(i5.inOpenClosedStarRange(i1, i4)); 
        assertFalse(i6.inOpenClosedStarRange(i1, i4)); 
        assertFalse(i7.inOpenClosedStarRange(i1, i4)); 

        // Corner cases from lead comment in ID.java
        assertFalse(i0.inOpenClosedStarRange(i0, i1)); 
        assertTrue (i1.inOpenClosedStarRange(i0, i1)); 
        assertFalse(i2.inOpenClosedStarRange(i0, i1)); 

        assertFalse(i0.inOpenClosedStarRange(i0, i0)); 
        assertFalse(i1.inOpenClosedStarRange(i0, i0)); 
        assertFalse(i2.inOpenClosedStarRange(i0, i0)); 

        assertTrue (i7.inOpenClosedStarRange(i0, i7)); 
        assertFalse(i0.inOpenClosedStarRange(i0, i7)); 
        assertTrue (i1.inOpenClosedStarRange(i0, i7)); 
        assertTrue (i2.inOpenClosedStarRange(i0, i7)); 
    }

    @Test
    public void testInOpenClosedRange() {
        assertFalse(i0.inOpenClosedRange(i1, i4)); 
        assertFalse(i1.inOpenClosedRange(i1, i4)); 
        assertTrue (i2.inOpenClosedRange(i1, i4)); 
        assertTrue (i3.inOpenClosedRange(i1, i4)); 
        assertTrue (i4.inOpenClosedRange(i1, i4)); 
        assertFalse(i5.inOpenClosedRange(i1, i4)); 
        assertFalse(i6.inOpenClosedRange(i1, i4)); 
        assertFalse(i7.inOpenClosedRange(i1, i4)); 

        // Corner cases from lead comment in ID.java
        assertFalse(i0.inOpenClosedRange(i0, i1)); 
        assertTrue (i1.inOpenClosedRange(i0, i1)); 
        assertFalse(i2.inOpenClosedRange(i0, i1)); 

        assertTrue (i0.inOpenClosedRange(i0, i0)); 
        assertTrue (i1.inOpenClosedRange(i0, i0)); 
        assertTrue (i2.inOpenClosedRange(i0, i0)); 

        assertTrue (i7.inOpenClosedRange(i0, i7)); 
        assertFalse(i0.inOpenClosedRange(i0, i7)); 
        assertTrue (i1.inOpenClosedRange(i0, i7)); 
        assertTrue (i2.inOpenClosedRange(i0, i7)); 
    }

    @Test
    public void testInOpenOpenRange() {
        assertFalse(i0.inOpenOpenRange(i1, i4)); 
        assertFalse(i1.inOpenOpenRange(i1, i4)); 
        assertTrue (i2.inOpenOpenRange(i1, i4)); 
        assertTrue (i3.inOpenOpenRange(i1, i4)); 
        assertFalse(i4.inOpenOpenRange(i1, i4)); 
        assertFalse(i5.inOpenOpenRange(i1, i4)); 
        assertFalse(i6.inOpenOpenRange(i1, i4)); 
        assertFalse(i7.inOpenOpenRange(i1, i4)); 

        // Corner cases from lead comment in ID.java
        assertFalse(i7.inOpenOpenRange(i0, i2)); 
        assertFalse(i0.inOpenOpenRange(i0, i2)); 
        assertTrue (i1.inOpenOpenRange(i0, i2)); 
        assertFalse(i2.inOpenOpenRange(i0, i2)); 
        assertFalse(i3.inOpenOpenRange(i0, i2)); 

        assertFalse(i7.inOpenOpenRange(i0, i1)); 
        assertFalse(i0.inOpenOpenRange(i0, i1)); 
        assertFalse(i1.inOpenOpenRange(i0, i1)); 
        assertFalse(i2.inOpenOpenRange(i0, i1)); 

        assertTrue (i7.inOpenOpenRange(i0, i0)); 
        assertFalse(i0.inOpenOpenRange(i0, i0)); 
        assertTrue (i1.inOpenOpenRange(i0, i0)); 

        assertTrue (i6.inOpenOpenRange(i0, i7)); 
        assertFalse(i7.inOpenOpenRange(i0, i7)); 
        assertFalse(i0.inOpenOpenRange(i0, i7)); 
        assertTrue (i1.inOpenOpenRange(i0, i7)); 

    }
    @Test
    public void testInClosedOpenStarRange() {
        assertFalse(i0.inClosedOpenStarRange(i1, i4)); 
        assertTrue (i1.inClosedOpenStarRange(i1, i4)); 
        assertTrue (i2.inClosedOpenStarRange(i1, i4)); 
        assertTrue (i3.inClosedOpenStarRange(i1, i4)); 
        assertFalse(i4.inClosedOpenStarRange(i1, i4)); 
        assertFalse(i5.inClosedOpenStarRange(i1, i4)); 
        assertFalse(i6.inClosedOpenStarRange(i1, i4)); 
        assertFalse(i7.inClosedOpenStarRange(i1, i4)); 

        // Corner cases from lead comment in ID.java
        assertFalse(i7.inClosedOpenStarRange(i0, i1)); 
        assertTrue (i0.inClosedOpenStarRange(i0, i1)); 
        assertFalse(i1.inClosedOpenStarRange(i0, i1)); 
        assertFalse(i2.inClosedOpenStarRange(i0, i1)); 

        assertFalse (i0.inClosedOpenStarRange(i0, i0)); 
        assertFalse (i1.inClosedOpenStarRange(i0, i0)); 
        assertFalse (i2.inClosedOpenStarRange(i0, i0)); 

        assertTrue (i6.inClosedOpenStarRange(i0, i7)); 
        assertFalse(i7.inClosedOpenStarRange(i0, i7)); 
        assertTrue (i0.inClosedOpenStarRange(i0, i7)); 
        assertTrue (i1.inClosedOpenStarRange(i0, i7)); 
    }

    @Test
    public void testInClosedOpenRange() {
        assertFalse(i0.inClosedOpenRange(i1, i4)); 
        assertTrue (i1.inClosedOpenRange(i1, i4)); 
        assertTrue (i2.inClosedOpenRange(i1, i4)); 
        assertTrue (i3.inClosedOpenRange(i1, i4)); 
        assertFalse(i4.inClosedOpenRange(i1, i4)); 
        assertFalse(i5.inClosedOpenRange(i1, i4)); 
        assertFalse(i6.inClosedOpenRange(i1, i4)); 
        assertFalse(i7.inClosedOpenRange(i1, i4)); 

        // Corner cases from lead comment in ID.java
        assertFalse(i7.inClosedOpenRange(i0, i1)); 
        assertTrue (i0.inClosedOpenRange(i0, i1)); 
        assertFalse(i1.inClosedOpenRange(i0, i1)); 
        assertFalse(i2.inClosedOpenRange(i0, i1)); 

        assertTrue (i0.inClosedOpenRange(i0, i0)); 
        assertTrue (i1.inClosedOpenRange(i0, i0)); 
        assertTrue (i2.inClosedOpenRange(i0, i0)); 

        assertTrue (i6.inClosedOpenRange(i0, i7)); 
        assertFalse(i7.inClosedOpenRange(i0, i7)); 
        assertTrue (i0.inClosedOpenRange(i0, i7)); 
        assertTrue (i1.inClosedOpenRange(i0, i7)); 
    }
}
