/*
 * MonitorImp.java
 *
 * @author  Paul Jackson
 *
 * Test master creates a single instance of this class and, in a multi-process
 * test, registers it with a Java RMI Registry.  Instance serves 2 purposes
 *
 * 1. It provides a way for test slaves to locate the test master on startup
 * 2. It formats and prints logging messages generated in test slave processes.
 * 
 */

package dhash;

import java.rmi.Remote;
import java.rmi.RemoteException;

import java.util.Collections;
import java.util.Map;
import java.util.HashMap;

//==EXTRA IMPORTS FOR SOLUTION ===============================================
//Keep this banner


public class MonitorImp implements Monitor {

    //== Static fields / initializers =========================================

    // See Serializable API docs. UnicastRemoteObject implements it.
    private static final long serialVersionUID = 42L;

    //== Instance fields ======================================================
    
    private Map<String,Object> map; // For key, value pairs

    //== Constructors =========================================================

    public MonitorImp() {
        map = Collections.synchronizedMap(new HashMap<String,Object>());
    }
    
    //== Methods ==============================================================

    // Non-remote methods

    public Object get(String key) {
        return map.get(key);
    }
    
    public int mapSize() {
        return map.size();
    }

    // Remotely accessible methods

    public void put(String key, Object val) {
        map.put(key, val);
    }

    //== SOLUTION CODE =======================================================
    //Keep this banner
    //If any of above class members need modifying, please move them to below
    //this banner.
    
    public void publish(String domain, String messageKind, String message) {

        String outStr = "";
        if (! domain.equals("")) outStr += "[" + domain + "] ";
        outStr += messageKind + " " + message;
        
        System.out.println(outStr); 
                           
    }


}
